// signal_analyzer.c : process example file and extract packets
// http://liquidsdr.org/blog/liquid-dsp-1.3.0/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <liquid/liquid.h>

// user-defined callback function
static int callback(unsigned char *  _header,
                    int              _header_valid,
                    unsigned char *  _payload,
                    unsigned int     _payload_len,
                    int              _payload_valid,
                    framesyncstats_s _stats,
                    void *           _userdata)
{
    printf("*** callback, header %s, payload %s (%u bytes) ***\n",
        _header_valid  ? "pass" : "FAIL",
        _payload_valid ? "pass" : "FAIL",
        _payload_len);
    framesyncstats_print(&_stats);

    // update counters and return
    unsigned int * counters = (unsigned int*)_userdata;
    counters[0] ++;                                   // number of frames detected
    counters[1] += _payload_valid ?            1 : 0; // number of frames valid
    counters[2] += _payload_valid ? _payload_len : 0; // number of bytes valid
    return 0;
}

int main(int argc, char *argv[])
{
    // options
    float fc            = 0.237f;       // center frequency
    float rate          = 7.2280;       // over-sampling rate
    char  filename[256] = "signal.dat"; // input filename
    if (argc > 1)
        strncpy(filename,argv[1],256);

    // create flexframesync object
    unsigned int counters[3] = {0,0,0}; // detects, frames, bytes
    flexframesync fs = flexframesync_create(callback,counters);

    // create oscillator for mixing to baseband
    nco_crcf nco = nco_crcf_create(LIQUID_VCO);
    nco_crcf_set_frequency(nco,2*3.14159f*fc);

    // create efficient resampler object
    msresamp_crcf resamp = msresamp_crcf_create(1.0f/rate, 80.0f);

    // receive frame in blocks
    unsigned int  buf_len = 256;
    float complex buf_file  [buf_len];  // buffer: samples from file
    float complex buf_nco   [buf_len];  // buffer: mixer output
    float complex buf_resamp[buf_len];  // buffer: resampler output

    // try to load input file
    FILE * fid_dat = fopen(filename,"r");
    if (fid_dat == NULL) {
        fprintf(stderr,"error: could not open '%s' for reading\n", filename);
        exit(1);
    }

    // process entire file
    while (!feof(fid_dat)) {
        // read up to buf_len samples from buffer
        int rc = fread(buf_file, sizeof(float complex), buf_len, fid_dat);

        // mix down to baseband
        nco_crcf_mix_block_down(nco, buf_file, buf_nco, rc);

        // resample
        unsigned int num_resamp = 0;
        msresamp_crcf_execute(resamp, buf_nco, rc, buf_resamp, &num_resamp);

        // push through frame synchronizer
        flexframesync_execute(fs, buf_resamp, num_resamp);
    }
    fclose(fid_dat);

    // destroy allocated objects
    flexframesync_destroy(fs);
    nco_crcf_destroy     (nco);
    msresamp_crcf_destroy(resamp);

    // print results
    printf("detected %u frames (%u valid), %u total valid bytes\n",
            counters[0], counters[1], counters[2]);
    return 0;
}
